package com.qmth.themis.business.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.themis.business.cache.bean.ExamCacheBean;
import com.qmth.themis.business.dao.TEOrgSummaryMapper;
import com.qmth.themis.business.entity.TBOrg;
import com.qmth.themis.business.entity.TEExamSummary;
import com.qmth.themis.business.entity.TEOrgSummary;
import com.qmth.themis.business.enums.OrgSummaryEnum;
import com.qmth.themis.business.service.TEExamService;
import com.qmth.themis.business.service.TEOrgSummaryService;
import com.qmth.themis.business.service.ThemisCacheService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.LinkedMultiValueMap;

import javax.annotation.Resource;
import java.util.List;
import java.util.StringJoiner;

/**
 * <p>
 * 超管统计表 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2023-10-27
 */
@Service
public class TEOrgSummaryServiceImpl extends ServiceImpl<TEOrgSummaryMapper, TEOrgSummary> implements TEOrgSummaryService {

    @Resource
    ThemisCacheService themisCacheService;

    @Resource
    TEExamService teExamService;

    @Resource
    TEOrgSummaryService teOrgSummaryService;

    /**
     * 保存机构统计信息公用
     *
     * @param orgId
     * @param examIdSet
     */
    @Override
    @Transactional
    public void orgSummary(Long orgId, List<Long> examIdSet) {
        int count = this.baseMapper.selectOrgSummaryCount(orgId);
        if (orgId.longValue() == 0) {//全局机构
            Integer onlineCount = this.baseMapper.orgSummaryByOnlineCount(examIdSet);
            Integer examCount = this.baseMapper.orgSummaryByExamCount(examIdSet);
            List<Integer> finishCountList = this.baseMapper.orgSummaryByFinishCountAndFinishStudentCount(examIdSet);
            TEOrgSummary teOrgSummary = new TEOrgSummary(orgId, onlineCount, examCount, finishCountList.get(0), finishCountList.get(1));
            if (count == 0) {
                teOrgSummaryService.saveOrgSummaryCommon(teOrgSummary);
            } else {
                teOrgSummaryService.updateOrgSummaryCommon(orgId, teOrgSummary);
            }
        } else {
            LinkedMultiValueMap<Long, Long> orgExamIdMap = new LinkedMultiValueMap<>();
            boolean reloadDb = false;
            //先查询考试批次统计缓存
            for (Long l : examIdSet) {
                List<TEExamSummary> teExamSummaryList = themisCacheService.addExamSummaryCache(l);
                if (!CollectionUtils.isEmpty(teExamSummaryList)) {//如果没有该批次的考试统计缓存信息，则自己从数据库取
                    ExamCacheBean examCacheBean = teExamService.getExamCacheBean(l);
                    TBOrg tbOrg = themisCacheService.addOrgCache(examCacheBean.getOrgId());
                    orgExamIdMap.add(tbOrg.getId(), l);
                } else {
                    reloadDb = true;
                    break;
                }
            }
            if (reloadDb) {//没有该批次的考试统计缓存信息，则从数据库取数据
                Integer onlineCount = this.baseMapper.orgSummaryByOnlineCount(examIdSet);
                Integer examCount = this.baseMapper.orgSummaryByExamCount(examIdSet);
                List<Integer> finishCountList = this.baseMapper.orgSummaryByFinishCountAndFinishStudentCount(examIdSet);
                TEOrgSummary teOrgSummary = new TEOrgSummary(orgId, onlineCount, examCount, finishCountList.get(0), finishCountList.get(1));
                if (count == 0) {
                    teOrgSummaryService.saveOrgSummaryCommon(teOrgSummary);
                } else {
                    teOrgSummaryService.updateOrgSummaryCommon(orgId, teOrgSummary);
                }
            } else {
                orgExamIdMap.forEach((k, v) -> {//从考试批次缓存信息里拿取在线人数、考试人数、完成考生数
                    Integer onlineCount = 0, examCount = 0, finishStudentCount = 0;
                    for (Long l : v) {
                        List<TEExamSummary> teExamSummaryList = themisCacheService.addExamSummaryCache(l);
                        for (TEExamSummary t : teExamSummaryList) {
                            onlineCount = onlineCount + t.getOnlineCount();
                            examCount = examCount + t.getExamCount();
                            finishStudentCount = finishStudentCount + t.getFinishCount();
                        }
                    }
                    Integer finishCount = this.baseMapper.orgSummaryByFinishCount(v);
                    TEOrgSummary teOrgSummary = new TEOrgSummary(orgId, onlineCount, examCount, finishCount, finishStudentCount);
                    if (count == 0) {
                        teOrgSummaryService.saveOrgSummaryCommon(teOrgSummary);
                    } else {
                        teOrgSummaryService.updateOrgSummaryCommon(orgId, teOrgSummary);
                    }
                });
            }
        }
    }

    /**
     * 保存机构统计信息
     *
     * @param teOrgSummary
     */
    @Override
    @Transactional
    public void saveOrgSummaryCommon(TEOrgSummary teOrgSummary) {
        StringJoiner stringJoinerFieldName = new StringJoiner(",");
        StringJoiner stringJoinerFieldValue = new StringJoiner(",");
        OrgSummaryEnum[] orgSummaryEnums = OrgSummaryEnum.values();
        for (int i = 0; i < orgSummaryEnums.length; i++) {
            stringJoinerFieldName.add(orgSummaryEnums[i].getCode());
        }
        stringJoinerFieldValue.add("'" + teOrgSummary.getOrgId() + "'")
                .add(teOrgSummary.getOnlineCount() + "")
                .add(teOrgSummary.getExamCount() + "")
                .add(teOrgSummary.getFinishCount() + "")
                .add(teOrgSummary.getFinishStudentCount() + "");
        this.baseMapper.saveOrgSummary(stringJoinerFieldName.toString(), stringJoinerFieldValue.toString());
    }

    /**
     * 更新机构统计信息
     *
     * @param teOrgSummary
     */
    @Override
    @Transactional
    public void updateOrgSummaryCommon(Long orgId, TEOrgSummary teOrgSummary) {
        StringJoiner stringJoinerFieldValue = new StringJoiner(",");
        OrgSummaryEnum[] orgSummaryEnums = OrgSummaryEnum.values();
        for (int i = 0; i < orgSummaryEnums.length; i++) {
            switch (orgSummaryEnums[i]) {
                case onlineCount:
                    stringJoinerFieldValue.add(orgSummaryEnums[i].getCode() + "=" + teOrgSummary.getOnlineCount());
                    break;
                case examCount:
                    stringJoinerFieldValue.add(orgSummaryEnums[i].getCode() + "=" + teOrgSummary.getExamCount());
                    break;
                case finishCount:
                    stringJoinerFieldValue.add(orgSummaryEnums[i].getCode() + "=" + teOrgSummary.getFinishCount());
                    break;
                case finishStudentCount:
                    stringJoinerFieldValue.add(orgSummaryEnums[i].getCode() + "=" + teOrgSummary.getFinishStudentCount());
                    break;
                default:
                    break;
            }
        }
        this.baseMapper.updateOrgSummary(orgId, stringJoinerFieldValue.toString());
    }

    /**
     * 更新全局机构考试完成数量和考生完成数量
     *
     * @param finishStudentCount
     */
    @Override
    @Transactional
    public void updateOrgZeroSummary(Integer finishStudentCount) {
        this.baseMapper.updateOrgZeroSummary(finishStudentCount);
    }
}